package com.icontrols.kakao.socket.wallpad;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.w3c.dom.Document;

import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.THINGS_INFO;
import com.icontrols.kakao.socket.utils.IPGenerator;
import com.icontrols.kakao.socket.utils.XMLConstants.REMOTE_KIND;
import com.icontrols.kakao.socket.utils.XMLGenerator;
import com.icontrols.kakao.socket.utils.iMapRequestSync;


@Service("wallpadMsgSender")
public class WallpadMsgSender {
	private final Logger logger = LoggerFactory.getLogger(WallpadMsgSender.class);
	
	@Resource(name="kakaoMsgHandler")
	KakaoMsgHandler kMsgHandler;
	
	/**
	 * 비동기로 여러 기기의 상태조회 메세지를 호출한다.
	 * @param dong
	 * @param ho
	 * @param endpointId
	 * @return
	 * @throws InterruptedException
	 */
	@Async("wallpadExecutor")
	public CompletableFuture<Map<String, Object>> getDeviceStatus(int dong, int ho, String endpointId) throws InterruptedException{
		Map<String, Object> retMap = new HashMap<String, Object> ();
		retMap.putAll(kMsgHandler.getDeviceStatus(dong, ho, endpointId));
		retMap.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, endpointId);
		
		return CompletableFuture.completedFuture(retMap);
	}
	
	/**
	 * 월패드에 구독/해제 메세지를 전송한다.
	 * @param param
	 * @return
	 */
	public Document remoteDeviceSubscription(Map<String, Object> param) {
		Document doc = null;
		
		try {
			int dong = Integer.parseInt((String)param.get("dong"));
			int ho = Integer.parseInt((String)param.get("ho"));
			
			String type = (String) param.get("type");
			String company = (String) param.get("company");
			
			int devNum = Integer.parseInt((String)param.get("devNum"));
			
			REMOTE_KIND kind = type.equalsIgnoreCase("subs")?REMOTE_KIND.msg_device_subscription:REMOTE_KIND.msg_device_unsubscription;
			
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeaderForSUBS(kind.getContents()))
			.append("      <req_company>").append(company).append("</req_company>\r\n")
			.append("      <dev_num>").append(devNum).append("</dev_num>")
			.append("      <model_id>wallpad</model_id>\r\n")
			.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			logger.info("[SUBS] - " + sb.toString());
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteDeviceList ERROR] - document null");
				return null;
			}
			
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("Subscription ERROR - " + e.getStackTrace());
		}
		
		return null;
	}
	
	/**
	 * 월패드에 인증 번호를 전송한다.
	 * @param param
	 * @return
	 */
	public Document remoteDeviceAuth(Map<String, Object> param) {
		Document doc = null;
		int dong = Integer.parseInt((String)param.get("dong"));
		int ho = Integer.parseInt((String)param.get("ho"));
		String code = (String)param.get("code");
		int expire_in = (int) param.get("expire_in");
		String alias = "Kakao";
		String twinid = UUID.randomUUID().toString();
		String targetIP = IPGenerator.getIPFromDongHo(dong, ho);
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeaderForNotice(REMOTE_KIND.remote_register_new_device_auth_number.getContents()))
			.append("      <home_info dong=\"").append(dong).append("\" ho=\"").append(ho).append("\" ip=\"").append(targetIP).append("\" type=\"main\"/>\r\n")
			.append("      <device_info alias=\"" + alias + "\" twinid=\"" + twinid)
			.append("\" auth_number=\"" + code + "\" />\r\n")
			.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteDeviceList ERROR] - document null");
				return null;
			}
			
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("[remoteDeviceList ERROR] - " + e.getMessage());
			return null;
		}
	}
	
	public Document remoteRegisterNewDevice(Map<String, Object> param) {
		Document doc = null;
		int dong = Integer.parseInt((String)param.get("dong"));
		int ho = Integer.parseInt((String)param.get("ho"));

		String alias = "KAKAO HOME";
		String twinid = UUID.randomUUID().toString();
		String targetIP = IPGenerator.getIPFromDongHo(dong, ho);
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeaderForNotice(REMOTE_KIND.remote_register_new_device_auth_number.getContents()))
			.append("      <home_info dong=\"").append(dong).append("\" ho=\"")
			.append(ho).append("\" ip=\"").append(targetIP).append("\" type=\"main\"/>\r\n")
			.append("      <device_info alias=\"" + alias + "\" twinid=\"" + twinid).append("\" />\r\n")
			.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteDeviceList ERROR] - document null");
				return null;
			}
			
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("[remoteDeviceList ERROR] - " + e.getMessage());
			return null;
		}
	}
	
	/**
	 * 월패드와 연결된 기기목록을 요청한다 
	 * @param dong
	 * @param ho
	 * @return
	 */
	public Document remoteDeviceList(int dong, int ho) {
		Document doc = null;
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho,REMOTE_KIND.renew_remote_get_home_devices.getContents()))
				.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteDeviceList ERROR] - document null");
				return null;
			}
			
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("[remoteDeviceList ERROR] - " + e.getMessage());
			return null;
		}
	}
	
	/**
	 * 월패드와 연결된 기기들의 전체 상태를 요청한다.
	 * @param dong
	 * @param ho
	 * @return
	 */
	public Document remoteDeviceStatusList(int dong, int ho) {
		Document doc = null;
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho,REMOTE_KIND.msg_total_devices_status.getContents()))
				.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteDeviceList ERROR] - document null");
				return null;
			}
			
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("[remoteDeviceList ERROR] - " + e.getMessage());
			return null;
		}
	}
	
	/**
	 * 월패드에 디바이스의 상태정보를 조회하는 XML을 전송하고 결과 Document를반환한다.
	 * @param dong
	 * @param ho
	 * @param modelId
	 * @param cirNum
	 * @return
	 */
	public Document remoteDeviceStatus(int dong, int ho, String modelId, int cirNum) {
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);

			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho,REMOTE_KIND.msg_home_device_get_status.getContents()))
			.append("      <model_id>").append(modelId).append("</model_id>\r\n")
			.append("      <dev_num>").append(cirNum).append("</dev_num>\r\n")
			.append("      <command_id>").append(UUID.randomUUID().toString()).append("</command_id>\r\n")
			.append(XMLGenerator.MakeXMLTail());
			
			xmlData = sb.toString();

			int bRet = imapSocket.doWork(xmlData);
			logger.debug(" bRet : " + bRet);

			if (bRet == 0) {
				logger.debug(" xml : " + imapSocket.getResponseXML());
				return imapSocket.getDocument();
			}
		}catch(Exception e) {
			return null;
		}
				
		return null;
	}
	
	/**
	 * 월패드에 디바이스의 상태를 번경하는 XML을 전송하고 결과 Document를 반환한다.
	 * @param dong
	 * @param ho
	 * @param modelId
	 * @param devNum
	 * @param action
	 * @return
	 */
	public Document remoteDeviceControl(int dong, int ho, String modelId, int devNum, String action) {
		iMapRequestSync imapSocket = new iMapRequestSync(
				IPGenerator.getIPFromDongHo(dong, ho),
				SERVERINFO.clientNewPort);

		String xmlData = null;
		StringBuffer sb = new StringBuffer();
		
		if(modelId.toLowerCase().contains(THINGS_INFO.THINGS.ELEVATOR.getThingsName().toLowerCase())) {
			if(!action.equalsIgnoreCase("down")) 
				return null;
			
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho, REMOTE_KIND.msg_elevator_call.getContents()))
			.append(XMLGenerator.MakeXMLTail());			
		}else {
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho, REMOTE_KIND.msg_home_device_set_control.getContents()))
			.append("      <req_company>kakao</req_company>\r\n")
			.append("      <model_id>").append(modelId).append("</model_id>\r\n")
			.append("      <dev_num>").append(devNum).append( "</dev_num>\r\n")
			.append("      <action>").append(action).append("</action>\r\n")
			.append("      <command_id>").append(UUID.randomUUID().toString()).append("</command_id>\r\n")
			.append(XMLGenerator.MakeXMLTail());
		}
		
		xmlData = sb.toString();

		int bRet = imapSocket.doWork(xmlData);
		
		logger.debug("xml : -----------  ");
		logger.debug(imapSocket.getResponseXML());
		
		return bRet == -1 ? null : imapSocket.getDocument();
	}
	
	/**
	 * 해당 세대의 주차위치 정보를 조회합니.
	 * @param dong
	 * @param ho
	 * @return
	 */
	public Document remoteGetParkingList(int dong, int ho) {
		Document doc = null;
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho,REMOTE_KIND.msg_car_parking_list.getContents()))
				.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteDeviceList ERROR] - document null");
				return null;
			}
			
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("[remoteDeviceList ERROR] - " + e.getMessage());
			return null;
		}
	}
	
	/**
	 * 에어컨의 속성을 조회합니다.
	 * @param dong
	 * @param ho
	 * @return
	 */
	public Document remoteGetAirconAttributes(int dong, int ho) {
		Document doc = null;
		try {
			iMapRequestSync imapSocket = new iMapRequestSync(
					IPGenerator.getIPFromDongHo(dong, ho),
					SERVERINFO.clientNewPort);
			
			String xmlData = null;
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeader(dong, ho,REMOTE_KIND.msg_aircon_attributes.getContents()))
				.append(XMLGenerator.MakeXMLTail());
		
			xmlData = sb.toString();
			
			int bRet = imapSocket.doWork(xmlData);
			
			if(bRet < 0) {
				logger.error("[remoteGetAirconAttributes ERROR] - document null");
				return null;
			}
			doc = imapSocket.getDocument();
			if(doc == null) {
				logger.error("doc is null - ");
			}
			return imapSocket.getDocument();
		}catch(Exception e) {
			logger.error("[remoteGetAirconAttributes ERROR] - " + e.getMessage());
			return null;
		}
	}
}
